-- Shag: Hair plug-in for 3D Studio MAX
-- Copyright (c) 1997, 2000, Dimension DESIGN Animation Group
-- 
-- File: ShagMgr.ms
-- Created by: Ivan Kolev
-- 
-- Global Shag parameter editing

global	strVersion = "1.0.2",
		CheckedColor = (color 0 255 255),
		rolMgr, rolCutters, rolParam, rolShade, rolDyn, rolLean,
		floatShagMgr,
		pShags = #(),
		desc,
		pAbsValues, pRelValues, pRanges,
		baseLen, baseDensPerArea, baseThick,
		strLoadFilename = ""


fn roundTo val n = (
	local mult = 10.0 ^ n
	(floor ((val * mult) + 0.5)) / mult
)


fn GetShagListFromEnv = (
	local i, n, temp, a

	n = numAtmospherics
	pa = #()
	for i=1 to n do
	(
		a = getAtmospheric i
		if ( classof a == ShagFur ) or ( classof a == ShagHair ) then
		(
			temp = #( 0, 1 )
			temp[1] = a
			append pa temp
		)
	)
	pa
)


fn GetShagNamesList pa = (
	local i, paNames

	paNames = #()
	for i=1 to pa.count do
		if pa[i][2] == 1 then
			append paNames ( "> " + pa[i][1].name )
		else
			append paNames ( "   " + pa[i][1].name )
	paNames
)


fn Str2Col v = (
	local i, r, g, b, c

	i = findString v " "
	r = 255 * ((substring v 1 (i-1)) as float)
	v = replace v 1 i ""

	i = findString v " "
	g = 255 * ((substring v 1 (i-1)) as float)
	v = replace v 1 i ""

	b = 255 * (v as float)

	c = color r g b
)


fn Str2Bool v = (
	v != "0"
)


fn findProp sect key = (
	local i, r = 0

	for i=1 to desc.count do if (desc[i][2] == sect) and (desc[i][3] == key) then r = i
	r
)


------------------------------------- Manager rollout -----------------------------

rollout rolMgr "Manager" (

listbox listShags "Shags:" offset:[0,5] height:5 align:#left width:200

button btnAbout " About " offset:[135,-100] width:49 height:20 align:left

button btnToggle "Toggle" offset:[208,0] width:49 height:20 align:#left
button btnInvert "Invert" offset:[261,-25] width:49 height:20 align:#left
button btnSetAll "All" offset:[208,0] width:49 height:20 align:#left
button btnSetNone "None" offset:[261,-25] width:49 height:20 align:#left
button btnSetFurs "Fur" offset:[208,0] width:49 height:20 align:#left
button btnSetHairs "Hair" offset:[261,-25] width:49 height:20 align:#left

button btnSyncShagList "Sync With Environment" offset:[0,0] width:200 height:20 align:#left

label lblSelectApply "Select Parameters:" align:#left offset:[0,10]
button btnSelectApplyAll "All +" width:50 height:20 align:#left offset:[92,-20]
button btnSelectApplyNone "None -" width:50 height:20 align:#left offset:[147,-25]
button btnSelectApplyInvert "Invert /" width:50 height:20 align:#left offset:[202,-25]
checkbox chkPrompt "Prompt" align:#left offset:[257,-23] checked:true

button btnGetParams "Get Parameters" offset:[0,5] width:95 height:20 align:#left
button btnSetParams "Set Parameters" offset:[105,-25] width:95 height:20 align:#left
button btnLoad "Load Parameters" offset:[210,-25] width:95 height:20 align:#left

label lblEditMode "Edit Mode" offset:[50,7] align:#left
radiobuttons radioEditMode labels:#("Absolute ", "Relative") offset:[110,-18] align:#left

button btnRelReset	" Reset " offset:[133,-23] width:50 height:20 align:left


-- checkbutton Interactive "Interactive Mode" width:140 height:30 highlightcolor:(color 0 255 255)
-- label InteractiveStatus


on rolMgr open do (

	pShags = GetShagListFromEnv()
	listShags.items = GetShagNamesList pShags
)

on btnSyncShagList pressed do (

	pShags = GetShagListFromEnv()
	listShags.items = GetShagNamesList pShags
)


on btnAbout pressed do (
	s = "Shag: Fur/Hair Manager, Version: "+ strVersion +
"\nBy Ivan Kolev\n\nDesigned for\nShag: Hair Version 1.01.00\n\n Dimension DESIGN Animation Group, 1999"
	messagebox s
)

on btnToggle pressed do (
	local i, t

	i = listShags.selection
	if i >= 1 and i <= listShags.items.count then
	(
		pShags[i][2] = 1 - pShags[i][2]
		listShags.items = GetShagNamesList pShags
	)
)


on btnInvert pressed do (
	local i

	for i=1 to pshags.count do pshags[i][2] = 1 - pshags[i][2]
	listShags.items = GetShagNamesList pShags
)


on btnSetAll pressed do (
	local i

	for i=1 to pshags.count do pshags[i][2] = 1
	listShags.items = GetShagNamesList pShags
)


on btnSetNone pressed do (
	local i

	for i=1 to pshags.count do pshags[i][2] = 0
	listShags.items = GetShagNamesList pShags
)


on btnSetFurs pressed do (
	local i

	for i=1 to pshags.count do
		if classof pshags[i][1] == ShagFur then pshags[i][2] = 1
		else pshags[i][2] = 0
	listShags.items = GetShagNamesList pShags
)


on btnSetHairs pressed do (
	local i

	for i=1 to pshags.count do
		if classof pshags[i][1] == ShagHair then pshags[i][2] = 1
		else pshags[i][2] = 0
	listShags.items = GetShagNamesList pShags
)


on btnSelectApplyAll pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) do
			desc[j][6].state = on
)


on btnSelectApplyNone pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) do
			desc[j][6].state = off
)


on btnSelectApplyInvert pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) do
			desc[j][6].state = not desc[j][6].state
)


on btnGetParams pressed do (
	local sel, i, kind, cont, prop, prompt

	sel = listShags.selection

	if sel >= 1 and sel <= listShags.items.count then
	(
		prompt = rolMgr.chkPrompt.state
		if prompt then
		(
			s = "Get selected parameters from highlighted Shag?"
			prompt = queryBox s title:"Shag Manager" beep:yes
		)
		else prompt = true

		if prompt then
		(
			shag = pShags[sel][1]
			if classof shag == ShagFur then kind = 1
			else kind = 2

			for i=1 to desc.count where
				( desc[i][4] > 0 ) and
				( desc[i][6].state == on ) and
				( desc[i][9] == kind or desc[i][9] == 3 ) do
			(
				cont = desc[i][5]
				prop = desc[i][1]
				case classof cont of
				(	SpinnerControl : if (desc[i][8]==1) then cont.value = (getproperty shag prop) / ticksPerFrame else cont.value = getproperty shag prop
					CheckBoxControl :
						if classof (getproperty shag prop) == Integer then
							cont.state = (getproperty shag prop) == 1
						else
							cont.state = getproperty shag prop
					RadioControl : cont.state = ( getproperty shag prop ) + 1
					ColorPickerControl : cont.color = getproperty shag prop
					UndefinedClass : i=i
					default : messagebox i
				)
			)
			-- get the scaling parameters
			baseLen = rolParam.spinLenMax.value
			baseDensPerArea = rolParam.spinDensPerArea.value
			baseThick = rolParam.spinThickMax.value
		)
	)
)


on btnSetParams pressed do (
	local i, j, kind, cont, prop
	
	prompt = rolMgr.chkPrompt.state
	if prompt then
	(
		s = "Set the selected parameters in the marked Shags?"
		prompt = queryBox s title:"Shag Manager" beep:yes
	)
	else prompt = true

	if prompt then

	undo on
	(

	for j=1 to listShags.items.count where pShags[j][2] == 1 do
	(
		shag = pShags[j][1]
		if classof shag == ShagFur then kind = 1
		else kind = 2

		if rolMgr.radioEditMode.state == 1 then			-- absolute mode
			for i=1 to desc.count where
				( desc[i][4] > 0 ) and
				( desc[i][6].state == on ) and
				( desc[i][9] == kind or desc[i][9] == 3 ) do
			(
				cont = desc[i][5]
				prop = desc[i][1]
				case classof cont of
				(	SpinnerControl : if (desc[i][8]==1) then setproperty shag prop (cont.value*ticksPerFrame) else setproperty shag prop cont.value
					CheckBoxControl :
						if classof (getproperty shag prop) == Integer then
							if cont.state then
								setproperty shag prop 1
							else
								setproperty shag prop 0
						else
							setproperty shag prop cont.state
					RadioControl : setproperty shag prop ( cont.state - 1 )
					ColorPickerControl : setproperty shag prop cont.color
					UndefinedClass : i=i
					default : messagebox i
				)
			)
		else											-- relative mode
			for i=1 to desc.count where
				( desc[i][4] > 0 ) and
				( desc[i][7] == 1 ) and
				( desc[i][6].state == on ) and
				( desc[i][9] == kind or desc[i][9] == 3 ) do
			(
				oldval = getproperty shag desc[i][1]
				setproperty shag desc[i][1] (oldval*desc[i][5].value)
			)
	)

	)	-- undo and if
)


on btnLoad pressed do (
	local file, ln, key, v, v1, sect, j, n, kind, cont

	if strLoadFilename == undefined then strLoadFilename = ""
	strLoadFilename = getOpenFileName caption:"Load Shag Parameters" filename:strLoadFilename types:"Shag: Fur files (*.fur)|*.fur|Shag: Hair files (*.hr)|*.hr|All files (*.*)|*.*|"

	prompt = rolMgr.chkPrompt.state
	if prompt then
	(
		s = "Load these parameters into the marked Shags and the manager?"
		prompt = queryBox s title:"Shag Manager" beep:yes
	)
	else prompt = true

	if prompt and strLoadFilename != undefined then
	(
		file = openFile strLoadFilename mode:"r"
		if ( file != undefined ) then while not eof file do (
			ln = readline file
			if ln[1]=="[" then
			(
				sect = substring ln 2 (ln.count-2)
			) else
			(
				n = findString ln "="
				if n != undefined then
				(
					key = substring ln 1 (n-1)
					v = substring ln (n+1) (ln.count-n)
					n = findProp sect key
					if n > 0 then
					(
						-- set the value in the selected shags
						for j=1 to listShags.items.count where
							pShags[j][2] == 1 do
						(
							shag = pShags[j][1]
							if classof shag == ShagFur then kind = 1
							else kind = 2
							if desc[n][9]==3 or desc[n][9] == kind then
							(
								case classof ( getproperty shag desc[n][1] ) of
								(
									Float : v1 = v as float
									Integer : v1 = v as integer
									BooleanClass : v1 = Str2Bool v
									Color : v1 = Str2Col v
								)
								setproperty shag desc[n][1] v1
							)
						)
						-- set the value in the manager interface
						if desc[n][4] > 0 and
							desc[n][6].state==on then
						(
							cont = desc[n][5]
							case classof cont of
							(	SpinnerControl : if (desc[n][8]==1) then cont.value = (v as float) / ticksPerFrame else cont.value = v as float
								CheckBoxControl : cont.state = Str2Bool v
								RadioControl : cont.state = (v as integer) + 1
								ColorPickerControl : cont.color = Str2Col v
								UndefinedClass : i=i
								default : messagebox i
							)
						)
					)
				)
			)
		)
		-- get the scaling parameters
		baseLen = rolParam.spinLenMax.value
		baseDensPerArea = rolParam.spinDensPerArea.value
		baseThick = rolParam.spinThickMax.value
	)
)


on radioEditMode changed newstate do (
	local i

	if newstate == 2 then
	(
		rolMgr.btnGetParams.enabled = false
		pAbsValues = #()
		pRanges = #()
		for i=1 to desc.count where desc[i][4] > 0 do
		if desc[i][7] == 0 then
		(
			desc[i][5].enabled = false
		)
		else
		(
			append pAbsValues desc[i][5].value
			append pRanges desc[i][5].range
			desc[i][5].range = [0,1000000,1]
			desc[i][5].value = pRelValues[1]
			deleteItem pRelValues 1
			-- get the scaling parameters
			baseLen = rolParam.spinLenMax.value
			baseDensPerArea = rolParam.spinDensPerArea.value
			baseThick = rolParam.spinThickMax.value
		)
	)
	else
	(
		rolMgr.btnGetParams.enabled = true
		pRelValues = #()
		for i=1 to desc.count where desc[i][4] > 0 do
		if desc[i][7] == 0 then
		(
			desc[i][5].enabled = true
		)
		else
		(
			append pRelValues desc[i][5].value
			desc[i][5].range = pRanges[1]
			desc[i][5].value = pAbsValues[1]
			-- get the scaling parameters
			baseLen = rolParam.spinLenMax.value
			baseDensPerArea = rolParam.spinDensPerArea.value
			baseThick = rolParam.spinThickMax.value
			deleteItem pAbsValues 1
			deleteItem pRanges 1
		)
	)
)


on btnRelReset pressed do (
	local i

	if rolMgr.radioEditMode == 0 then
		for i=1 to desc.count where desc[i][4] > 0 and desc[i][7] == 1 do
			desc[i][5].value = 1
	rolParam.spinScale.value = 1
	-- get the scaling parameters
	baseLen = rolParam.spinLenMax.value
	baseDensPerArea = rolParam.spinDensPerArea.value
	baseThick = rolParam.spinThickMax.value
)

)	-- end rollout


------------------------------------- Cutters rollout ---------------------------

rollout rolCutters "Cutters, Auto Influence Mixing" (

checkbox chkModCutsEnabled "Modeling Cutters Enabled" offset:[0,4] align:#left
checkbutton chkModCutsEnabledApply offset:[168,-18] width:10 height:10 align:#left highlightcolor:CheckedColor
checkbox chkRestrCutsEnabled "Restricting Cutters Enabled" offset:[0,8] align:#left
checkbutton chkRestrCutsEnabledApply offset:[168,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

spinner spinAutoInfMix "Auto Influence Mixing  " offset:[0,8] range:[0,1,0.5] type:#float fieldwidth:45 align:#left
checkbutton chkAutoInfMixApply offset:[168,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

label lblEmpty " " height:2

)

---------------------------------- Parameters rollout -----------------------------

rollout rolParam "Parameters" (

spinner spinScale "Scale                " range:[0.001,1000000,1] type:#float fieldwidth:45 align:#left

button btnSelectApplyAll "+" width:30 height:20 align:#left offset:[175,-22]
button btnSelectApplyNone "-" width:30 height:20 align:#left offset:[205,-25]
button btnSelectApplyInvert "/" width:30 height:20 align:#left offset:[235,-25]
button btnSelectApplyScale "Scale" width:40 height:20 align:#left offset:[265,-25]

group "Length ------------------------------------------ Density" (

	spinner spinLenMax "Maximum           " range:[0,1000000,4] type:#float fieldwidth:45 align:#left
	checkbutton chkLenMaxApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinLenRand "Random Factor " range:[0,1,0] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkLenRandApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkLenFromMat offset:[12,5] align:#left
	checkbutton chkLenFromMatApply offset:[0,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinLenMat "SubMatID" range:[1,65535,1] offset:[28,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkLenMatApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	radiobuttons radioDensityType labels:#("Absolute", "Per Area") offset:[160,-59] align:#left default:2 columns:1
	checkbutton chkDensityTypeApply offset:[224,-23] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinDensAbs range:[0.001,1000000,1] offset:[235,-30] type:#float fieldwidth:45 align:#left
	checkbutton chkDensAbsApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinDensPerArea range:[0.001,1000000,0.2] offset:[235,1] type:#float fieldwidth:45 align:#left
	checkbutton chkDensPerAreaApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkDensFromMat offset:[172,5] align:#left
	checkbutton chkDensFromMatApply offset:[160,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinDensMat "SubMatID" range:[1,65535,1] offset:[187,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkDensMatApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty " " height:1
)


button btnSelectApplyLengthAll "+" width:18 height:18 align:#left offset:[94,-95]
button btnSelectApplyLengthNone "-" width:18 height:18 align:#left offset:[111,-23]
button btnSelectApplyLengthInvert "/" width:18 height:18 align:#left offset:[129,-23]

button btnSelectApplyDensAll "+" width:18 height:18 align:#left offset:[254,-23]
button btnSelectApplyDensNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyDensInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty4 " " offset:[0,53]


group "Thickness ------------------------------------- Curliness" (

	spinner spinThickMax "Maximum           " range:[0,1000000,0.2] type:#float fieldwidth:45 align:#left
	checkbutton chkThickMaxApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkThickFromMat offset:[12,5] align:#left
	checkbutton chkThickFromMatApply offset:[0,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinThickMat "SubMatID" range:[1,65535,1] offset:[28,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkThickMatApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinThickRand "Random Factor " range:[0,1,0] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkThickRandApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkLinkToLenRand "Link To Length Random" offset:[12,5] align:#left
	checkbutton chkLinkToLenRandApply offset:[0,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinShadowFactor "Shadow Factor " range:[0,1,1] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkShadowFactorApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblSection "Section" offset:[0,5] align:#left
	radiobuttons radioSection labels:#("Round", "Flat") offset:[42,-18] align:#left default:1 columns:2
	checkbutton chkSectionApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkCurlEnabled "Enabled" offset:[172,-131] align:#left
	checkbutton chkCurlEnabledApply offset:[160,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinCurlRadius "Radius              " range:[0,1000000,1] offset:[160,12] type:#float fieldwidth:45 align:#left
	checkbutton chkCurlRadiusApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkCurlRadiusFromMat offset:[172,5] align:#left
	checkbutton chkCurlRadiusFromMatApply offset:[160,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinCurlRadiusMat "SubMatID" range:[1,65535,1] offset:[187,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkCurlRadiusMatApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinCurlTurns "Turns                " range:[0,1000000,1] offset:[160,18] type:#float fieldwidth:45 align:#left
	checkbutton chkCurlTurnsApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkCurlTurnsFromMat offset:[172,5] align:#left
	checkbutton chkCurlTurnsFromMatApply offset:[160,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinCurlTurnsMat "SubMatID" range:[1,65535,1] offset:[187,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkCurlTurnsMatApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty2 " " height:1
)


button btnSelectApplyThickAll "+" width:18 height:18 align:#left offset:[94,-163]
button btnSelectApplyThickNone "-" width:18 height:18 align:#left offset:[111,-23]
button btnSelectApplyThickInvert "/" width:18 height:18 align:#left offset:[129,-23]

button btnSelectApplyCurlAll "+" width:18 height:18 align:#left offset:[254,-23]
button btnSelectApplyCurlNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyCurlInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty5 " " offset:[0,121]


group "Clumping" (

	spinner spinClumpAmount "Amount             " range:[0,1,0] type:#float fieldwidth:45 align:#left
	checkbutton chkClumpAmountApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkClumpAmountFromMat offset:[12,3] align:#left
	checkbutton chkClumpAmountFromMatApply offset:[0,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinClumpAmountMat "SubMatID" range:[1,65535,1] offset:[28,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkClumpAmountMatApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinClumpRadius "Radius              " range:[0,1000000,10] offset:[0,8] type:#float fieldwidth:45 align:#left
	checkbutton chkClumpRadiusApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbox chkClumpRadiusFromMat offset:[12,3] align:#left
	checkbutton chkClumpRadiusFromMatApply offset:[0,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinClumpRadiusMat "SubMatID" range:[1,65535,1] offset:[28,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkClumpRadiusMatApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblAttractors "Attractors:" offset:[160,-88] align:#left
	checkbutton chkAttractorsApply offset:[212,-16] width:10 height:10 align:#left highlightcolor:CheckedColor

	radiobuttons radioAttractors labels:#( "Vertex Strands", "Absolute", "Per Area", "Percent" ) offset:[160,3] default:1 columns:1 align:#left

	spinner spinClumpPerFace range:[0,1000000,0.2] offset:[237,-50] type:#float fieldwidth:45 align:#left
	checkbutton chkClumpPerFaceApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinClumpPerArea range:[0,1000000,0.05] offset:[237,0] type:#float fieldwidth:45 align:#left
	checkbutton chkClumpPerAreaApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinClumpPercent range:[0,1,0.05] offset:[237,0] type:#float fieldwidth:45 align:#left
	checkbutton chkClumpPercentApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty3 " " height:2
)


button btnSelectApplyClumpAll "+" width:18 height:18 align:#left offset:[254,-121]
button btnSelectApplyClumpNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyClumpInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty6 " " offset:[0,80]

group "Fix To Emitter" (

	checkbox chkFixShapeToEmitter "Enabled" offset:[12,0] align:#left
	checkbutton chkFixShapeToEmitterApply offset:[0,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinRefFrame "Reference Frame" range:[-10000000,10000000,0] offset:[155,-16] type:#integer fieldwidth:45 align:#left
	checkbutton chkRefFrameApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty7 " " height:1
)


group "Orientation-------------------------------------Random" (

	label lblAxis "Axis" align:#left offset:[1,0]
	radiobuttons radioOrientAxis labels:#( "X", "Y", "Z" ) offset:[45,-18] default:3 columns:3 align:#left
	checkbutton chkOrientAxisApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinOrientAngle "Angle                 " range:[-1000000,1000000,-90] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkOrientAngleApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinOrientRand "Random Factor  " range:[0,1,0] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkOrientRandApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinMiddleRand "Middle Strands  " range:[0,1,0] offset:[160,-63] type:#float fieldwidth:45 align:#left
	checkbutton chkMiddleRandApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinVertexRand "Vertex Strands  " range:[0,1,0] offset:[160,5] type:#float fieldwidth:45 align:#left
	checkbutton chkVertexRandApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinRandSeed "Seed                 " range:[0,10000,0] offset:[160,5] type:#integer fieldwidth:45 align:#left
	checkbutton chkRandSeedApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty8 " " height:1
)

button btnSelectApplyOrientAll "+" width:18 height:18 align:#left offset:[94,-95]
button btnSelectApplyOrientNone "-" width:18 height:18 align:#left offset:[111,-23]
button btnSelectApplyOrientInvert "/" width:18 height:18 align:#left offset:[129,-23]

button btnSelectApplyRandAll "+" width:18 height:18 align:#left offset:[254,-23]
button btnSelectApplyRandNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyRandInvert "/" width:18 height:18 align:#left offset:[290,-23]


on rolParam open do (
	-- get the scaling parameters
	baseLen = 4
	baseDensPerArea = 0.2
	baseThick = 0.2
)

on spinLenMax changed value do (
	baseLen = value
)


on spinDensPerArea changed value do (
	baseDensPerArea = value
)


on spinThickMax changed value do (
	baseThick = value
)


on spinScale changed newValue do (
	
	spinLenMax.value = baseLen * newValue
	spinDensPerArea.value = baseDensPerArea / ( newValue * newValue )
	spinThickMax.value = baseThick * newValue
)


on btnSelectApplyAll pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 2 ) do
			desc[j][6].state = on
)

on btnSelectApplyNone pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 2 ) do
			desc[j][6].state = off
)

on btnSelectApplyInvert pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 2 ) do
			desc[j][6].state = not desc[j][6].state
)

on btnSelectApplyScale pressed do (
	local j

	chkLenMaxApply.state = on
	chkDensPerAreaApply.state = on
	chkThickMaxApply.state = on
)


on btnSelectApplyLengthAll pressed do (

	chkLenMaxApply.state = on
	chkLenRandApply.state = on
	chkLenFromMatApply.state = on
	chkLenMatApply.state = on
)

on btnSelectApplyLengthNone pressed do (

	chkLenMaxApply.state = off
	chkLenRandApply.state = off
	chkLenFromMatApply.state = off
	chkLenMatApply.state = off
)

on btnSelectApplyLengthInvert pressed do (

	chkLenMaxApply.state = not chkLenMaxApply.state
	chkLenRandApply.state = not chkLenRandApply.state
	chkLenFromMatApply.state = not chkLenFromMatApply.state
	chkLenMatApply.state = not chkLenMatApply.state
)


on btnSelectApplyThickAll pressed do (

	chkThickMaxApply.state = on
	chkThickFromMatApply.state = on
	chkThickMatApply.state = on
	chkThickRandApply.state = on
	chkLinkToLenRandApply.state = on
	chkShadowFactorApply.state = on
	chkSectionApply.state = on
)

on btnSelectApplyThickNone pressed do (

	chkThickMaxApply.state = off
	chkThickFromMatApply.state = off
	chkThickMatApply.state = off
	chkThickRandApply.state = off
	chkLinkToLenRandApply.state = off
	chkShadowFactorApply.state = off
	chkSectionApply.state = off
)

on btnSelectApplyThickInvert pressed do (

	chkThickMaxApply.state = not chkThickMaxApply.state
	chkThickFromMatApply.state = not chkThickFromMatApply.state
	chkThickMatApply.state = not chkThickMatApply.state
	chkThickRandApply.state = not chkThickRandApply.state
	chkLinkToLenRandApply.state = not chkLinkToLenRandApply.state
	chkShadowFactorApply.state = not chkShadowFactorApply.state
	chkSectionApply.state = not chkSectionApply.state
)

on btnSelectApplyCurlAll pressed do (

	chkCurlEnabledApply.state = on
	chkCurlRadiusApply.state = on
	chkCurlRadiusFromMatApply.state = on
	chkCurlRadiusMatApply.state = on
	chkCurlTurnsApply.state = on
	chkCurlTurnsFromMatApply.state = on
	chkCurlTurnsMatApply.state = on
)

on btnSelectApplyCurlNone pressed do (

	chkCurlEnabledApply.state = off
	chkCurlRadiusApply.state = off
	chkCurlRadiusFromMatApply.state = off
	chkCurlRadiusMatApply.state = off
	chkCurlTurnsApply.state = off
	chkCurlTurnsFromMatApply.state = off
	chkCurlTurnsMatApply.state = off
)

on btnSelectApplyCurlInvert pressed do (

	chkCurlEnabledApply.state = not chkCurlEnabledApply.state
	chkCurlRadiusApply.state = not chkCurlRadiusApply.state
	chkCurlRadiusFromMatApply.state = not chkCurlRadiusFromMatApply.state
	chkCurlRadiusMatApply.state = not chkCurlRadiusMatApply.state
	chkCurlTurnsApply.state = not chkCurlTurnsApply.state
	chkCurlTurnsFromMatApply.state = not chkCurlTurnsFromMatApply.state
	chkCurlTurnsMatApply.state = not chkCurlTurnsMatApply.state
)


on btnSelectApplyClumpAll pressed do (

	chkClumpAmountApply.state = on
	chkClumpAmountFromMatApply.state = on
	chkClumpAmountMatApply.state = on
	chkClumpRadiusApply.state = on
	chkClumpRadiusFromMatApply.state = on
	chkClumpRadiusMatApply.state = on
	chkAttractorsApply.state = on
	chkClumpPerFaceApply.state = on
	chkClumpPerAreaApply.state = on
	chkClumpPercentApply.state = on
)

on btnSelectApplyClumpNone pressed do (

	chkClumpAmountApply.state = off
	chkClumpAmountFromMatApply.state = off
	chkClumpAmountMatApply.state = off
	chkClumpRadiusApply.state = off
	chkClumpRadiusFromMatApply.state = off
	chkClumpRadiusMatApply.state = off
	chkAttractorsApply.state = off
	chkClumpPerFaceApply.state = off
	chkClumpPerAreaApply.state = off
	chkClumpPercentApply.state = off
)

on btnSelectApplyClumpInvert pressed do (

	chkClumpAmountApply.state = not chkClumpAmountApply.state
	chkClumpAmountFromMatApply.state = not chkClumpAmountFromMatApply.state
	chkClumpAmountMatApply.state = not chkClumpAmountMatApply.state
	chkClumpRadiusApply.state = not chkClumpRadiusApply.state
	chkClumpRadiusFromMatApply.state = not chkClumpRadiusFromMatApply.state
	chkClumpRadiusMatApply.state = not chkClumpRadiusMatApply.state
	chkAttractorsApply.state = not chkAttractorsApply.state
	chkClumpPerFaceApply.state = not chkClumpPerFaceApply.state
	chkClumpPerAreaApply.state = not chkClumpPerAreaApply.state
	chkClumpPercentApply.state = not chkClumpPercentApply.state
)


on btnSelectApplyOrientAll pressed do (

	chkOrientAxisApply.state = on
	chkOrientAngleApply.state = on
	chkOrientRandApply.state = on
)


on btnSelectApplyOrientNone pressed do (

	chkOrientAxisApply.state = off
	chkOrientAngleApply.state = off
	chkOrientRandApply.state = off
)


on btnSelectApplyOrientInvert pressed do (

	chkOrientAxisApply.state = not chkOrientAxisApply.state
	chkOrientAngleApply.state = not chkOrientAngleApply.state
	chkOrientRandApply.state = not chkOrientRandApply.state
)


on btnSelectApplyRandAll pressed do (

	chkMiddleRandApply.state = on
	chkVertexRandApply.state = on
	chkRandSeedApply.state = on
)


on btnSelectApplyRandNone pressed do (

	chkMiddleRandApply.state = off
	chkVertexRandApply.state = off
	chkRandSeedApply.state = off
)


on btnSelectApplyRandInvert pressed do (

	chkMiddleRandApply.state = not chkMiddleRandApply.state
	chkVertexRandApply.state = not chkVertexRandApply.state
	chkRandSeedApply.state = not chkRandSeedApply.state
)

)	-- end rollout


---------------------------------- Shading rollout -----------------------------

rollout rolShade "Shading, Geometry, Quality" (


button btnSelectApplyAll "+" width:30 height:20 align:#left offset:[215,-2]
button btnSelectApplyNone "-" width:30 height:20 align:#left offset:[245,-25]
button btnSelectApplyInvert "/" width:30 height:20 align:#left offset:[275,-25]

group "Color" (

	colorpicker colTip "Tip         " color:[153,153,153] fieldwidth:30 height:24 title:"Tip Color"
	checkbutton chkColTipApply offset:[30,-21] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbutton chkTipFromMatApply offset:[85,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkTipFromMat offset:[97,-17] align:#left

	spinner spinTipMat "From SubMatID" range:[1,65535,1] offset:[113,-19] type:#integer fieldwidth:45 align:#left
	checkbutton chkTipMatApply offset:[248,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	colorpicker colBase "Base      " color:[153,153,153] offset:[0,20] fieldwidth:30 height:24 title:"Tip Color"
	checkbutton chkColBaseApply offset:[30,-21] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbutton chkBaseFromMatApply offset:[85,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkBaseFromMat offset:[97,-17] align:#left

	spinner spinBaseMat "From SubMatID" range:[1,65535,1] offset:[113,-19] type:#integer fieldwidth:45 align:#left
	checkbutton chkBaseMatApply offset:[248,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinColBias "Bias" range:[-1,1,0] offset:[213,-39] type:#float fieldwidth:45 align:#left
	checkbutton chkColBiasApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinColRand "Random Factor" range:[0,1,0] offset:[160,26] type:#float fieldwidth:45 align:#left
	checkbutton chkColRandApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblMatID "Shag Material ID Assignment:" align:#left

	radiobuttons radioMatIDType labels:#("Fixed Mtl ID","Get Emitter's Mtl IDs") default:1 columns:1 align:#left
	checkbutton chkMatIDTypeApply offset:[115,-25] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinMatID range:[1,65535,1] offset:[130,-27] type:#integer fieldwidth:45 align:#left
	checkbutton chkMatIDApply offset:[192,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty " " offset:[0,14] height:1
)

button btnSelectApplyColorAll "+" width:18 height:18 align:#left offset:[254,-170]
button btnSelectApplyColorNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyColorInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty2 " " offset:[0,128]

group "Shading" (

	checkbutton chkShadeBezSegsApply offset:[0,1] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkShadeBezSegs "Shade Bezier Segments" offset:[12,-17] align:#left

	checkbutton chkShadeMidApply offset:[0,5] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkShadeMid "Shade Middle Strands" offset:[12,-17] align:#left

	spinner spinWidthSamples "Width Samples " range:[1,100,1] offset:[160,-43] type:#integer fieldwidth:45 align:#left
	checkbutton chkWidthSamplesApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbutton chkFilteringApply offset:[160,6] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkFiltering "Filtering" offset:[172,-17] align:#left

	label lblEmpty3 " " offset:[0,0] height:1
)

button btnSelectApplyShadingAll "+" width:18 height:18 align:#left offset:[254,-73]
button btnSelectApplyShadingNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyShadingInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty4 " " offset:[0,42] height:1

group "Geometry" (

	checkbutton chkGeomEnableApply offset:[0,1] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkGeomEnable "Enable" offset:[12,-17] align:#left

	checkbutton chkGeomSmoothApply offset:[80,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkGeomSmooth "Smooth" offset:[93,-17] align:#left checked:true

	spinner spinGeomSides "Sides                 " range:[2,100,3] offset:[0,5] type:#integer fieldwidth:45 align:#left
	checkbutton chkGeomSidesApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinGeomMinThick "Thinner Than    " range:[0,1000000,1] offset:[160,-43] type:#float fieldwidth:45 align:#left
	checkbutton chkGeomMinThickApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	radiobuttons radioGeomThin labels:#("Ignore","Render") offset:[160,7] default:1 columns:2 align:#left
	checkbutton chkGeomThinApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty5 " " height:1
)

button btnSelectApplyGeomAll "+" width:18 height:18 align:#left offset:[254,-73]
button btnSelectApplyGeomNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyGeomInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty6 " " offset:[0,42] height:1


group "Picture Quality-------------------------------Strand Detail" (

	spinner spinQuality "Quality/Memory " range:[0,10,0] offset:[0,1] type:#float fieldwidth:45 align:#left
	checkbutton chkQualityApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinViewDens "Viewport Density" range:[0,100,40] offset:[0,5] type:#integer fieldwidth:45 align:#left
	checkbutton chkViewDensApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinKnots "Knots                " range:[2,1000,5] offset:[160,-41] type:#integer fieldwidth:45 align:#left
	checkbutton chkKnotsApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinBezSteps "Bezier Steps     " range:[1,1000,5] offset:[160,5] type:#integer fieldwidth:45 align:#left
	checkbutton chkBezStepsApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty7 " " height:1
)



on btnSelectApplyAll pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 3 ) do
			desc[j][6].state = on
)

on btnSelectApplyNone pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 3 ) do
			desc[j][6].state = off
)

on btnSelectApplyInvert pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 3 ) do
			desc[j][6].state = not desc[j][6].state
)


on btnSelectApplyColorAll pressed do (

	chkColTipApply.state = on
	chkTipFromMatApply.state = on
	chkTipMatApply.state = on
	chkColBaseApply.state = on
	chkBaseFromMatApply.state = on
	chkBaseMatApply.state = on
	chkColBiasApply.state = on
	chkColRandApply.state = on
	chkMatIDTypeApply.state = on
	chkMatIDApply.state = on
)


on btnSelectApplyColorNone pressed do (

	chkColTipApply.state = off
	chkTipFromMatApply.state = off
	chkTipMatApply.state = off
	chkColBaseApply.state = off
	chkBaseFromMatApply.state = off
	chkBaseMatApply.state = off
	chkColBiasApply.state = off
	chkColRandApply.state = off
	chkMatIDTypeApply.state = off
	chkMatIDApply.state = off
)


on btnSelectApplyColorInvert pressed do (

	chkColTipApply.state = not chkColTipApply.state
	chkTipFromMatApply.state = not chkTipFromMatApply.state
	chkTipMatApply.state = not chkTipMatApply.state
	chkColBaseApply.state = not chkColBaseApply.state
	chkBaseFromMatApply.state = not chkBaseFromMatApply.state
	chkBaseMatApply.state = not chkBaseMatApply.state
	chkColBiasApply.state = not chkColBiasApply.state
	chkColRandApply.state = not chkColRandApply.state
	chkMatIDTypeApply.state = not chkMatIDTypeApply.state
	chkMatIDApply.state = not chkMatIDApply.state
)


on btnSelectApplyShadingAll pressed do (

	chkShadeBezSegsApply.state = on
	chkShadeMidApply.state = on
	chkWidthSamplesApply.state = on
	chkFilteringApply.state = on
)


on btnSelectApplyShadingNone pressed do (

	chkShadeBezSegsApply.state = off
	chkShadeMidApply.state = off
	chkWidthSamplesApply.state = off
	chkFilteringApply.state = off
)


on btnSelectApplyShadingInvert pressed do (

	chkShadeBezSegsApply.state = not chkShadeBezSegsApply.state
	chkShadeMidApply.state = not chkShadeMidApply.state
	chkWidthSamplesApply.state = not chkWidthSamplesApply.state
	chkFilteringApply.state = not chkFilteringApply.state
)

on btnSelectApplyGeomAll pressed do (

	chkGeomEnableApply.state = on
	chkGeomSmoothApply.state = on
	chkGeomSidesApply.state = on
	chkGeomMinThickApply.state = on
	chkGeomThinApply.state = on
)


on btnSelectApplyGeomNone pressed do (

	chkGeomEnableApply.state = off
	chkGeomSmoothApply.state = off
	chkGeomSidesApply.state = off
	chkGeomMinThickApply.state = off
	chkGeomThinApply.state = off
)


on btnSelectApplyGeomInvert pressed do (

	chkGeomEnableApply.state = not chkGeomEnableApply.state
	chkGeomSmoothApply.state = not chkGeomSmoothApply.state
	chkGeomSidesApply.state = not chkGeomSidesApply.state
	chkGeomMinThickApply.state = not chkGeomMinThickApply.state
	chkGeomThinApply.state = not chkGeomThinApply.state
)

)	-- end rollout


-------------------------------- Dynamics rollout --------------------------------

rollout rolDyn "Dynamics" (

button btnSelectApplyAll "+" width:30 height:20 align:#left offset:[215,-2]
button btnSelectApplyNone "-" width:30 height:20 align:#left offset:[245,-25]
button btnSelectApplyInvert "/" width:30 height:20 align:#left offset:[275,-25]

group "General" (

	label lblDynUsage "Usage: " offset:[0,-2] align:#left
	checkbutton chkDynUsageApply offset:[55,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	radiobuttons radioDynUsage labels:#("No Dynamics","On-The-Fly","Pre-calculated") offset:[0,1] default:1 columns:3 align:#left

	label lblDynMethod "Method: " offset:[0,2] align:#left
	checkbutton chkDynMethodApply offset:[55,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	radiobuttons radioDynMethod labels:#("Weighted       ","None (just calc velocity for IMBlur)") offset:[0,1] default:1 align:#left

	label lblEmpty " " height:1 offset:[0,-5]
)

group "Pre-Calculation" (
	
	spinner spinKeyThresh "Key Threshold  " range:[0,1000000,0.5] offset:[0,1] type:#float fieldwidth:45 align:#left
	checkbutton chkKeyThreshApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblKeyType "Key Type " offset:[0,2] align:#left
	checkbutton chkKeyTypeApply offset:[58,-16] width:10 height:10 align:#left highlightcolor:CheckedColor
	radiobuttons radioKeyType labels:#("Linear","Bezier") offset:[75,-17] default:1 columns:3 align:#left

--	label lblEmpty2 " " height:1
)

group "Common Properties" (
	
	spinner spinStartFrame "Start Frame        " range:[-1000000,1000000,0] offset:[0,1] type:#integer fieldwidth:45 align:#left
	checkbutton chkStartFrameApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinEndFrame "End Frame         " range:[-1000000,1000000,100] offset:[0,5] type:#integer fieldwidth:45 align:#left
	checkbutton chkEndFrameApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinMass "Mass                  " range:[0,1000000,1] offset:[154,-40] type:#float fieldwidth:45 align:#left
	checkbutton chkMassApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinMaxVelo "Max Velocity      " range:[0,1000,0] offset:[154,5] type:#float fieldwidth:45 align:#left
	checkbutton chkMaxVeloApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbutton chkGravsEnableApply offset:[10,10] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkGravsEnable "Gravity" offset:[22,-17] align:#left checked:false

	checkbutton chkWindsEnableApply offset:[110,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkWindsEnable "Winds" offset:[122,-17] align:#left checked:false

	label lblEmpty3 " " height:1 offset:[0,-5]
)

group "Collision Properties" (

	checkbutton chkColsEnableApply offset:[0,2] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkColsEnable "Enabled " offset:[12,-17] align:#left checked:false

	checkbutton chkSegCollsApply offset:[0,8] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkSegColls "Segment Collisions" offset:[12,-17] align:#left checked:false

	checkbutton chkGetOutApply offset:[128,-17] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkGetOut "Get Hair out of Colliders at Start" offset:[140,-17] align:#left checked:false

	spinner spinColFrict "Sliding Friction    " range:[0,1,0.2] offset:[0,2] type:#float fieldwidth:45 align:#left
	checkbutton chkColFrictApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinColStaticFrict "Static Friction     " range:[0,1,0.4] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkColStaticFrictApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinColDist "Collision Distance" range:[0,1000000,2] offset:[154,-40] type:#float fieldwidth:45 align:#left
	checkbutton chkColDistApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinColDistRand "Random Factor   " range:[0,1,1] offset:[154,5] type:#float fieldwidth:45 align:#left
	checkbutton chkColDistRandApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty4 " " height:5 offset:[0,-5]
)


group "Weighted Method" (

	spinner spinSimpElast "Elasticity            " range:[0,1,0.2] offset:[0,2] type:#float fieldwidth:45 align:#left
	checkbutton chkSimpElast offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty5 " " height:2 offset:[0,-5]
)

on btnSelectApplyAll pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 4 ) do
			desc[j][6].state = on
)

on btnSelectApplyNone pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 4 ) do
			desc[j][6].state = off
)

on btnSelectApplyInvert pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 4 ) do
			desc[j][6].state = not desc[j][6].state
)

)
	
---------------------------- Leaning/Bending rollout -----------------------------

rollout rolLean "Leaning, Bending" (

button btnSelectApplyAll "+" width:30 height:20 align:#left offset:[215,-2]
button btnSelectApplyNone "-" width:30 height:20 align:#left offset:[245,-25]
button btnSelectApplyInvert "/" width:30 height:20 align:#left offset:[275,-25]

group "Leaning" (

	checkbutton chkLeanEnableApply offset:[0,1] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkLeanEnable "Enabled" offset:[12,-17] align:#left checked:true

	spinner spinLeanAmount "Amount              " range:[-1,1,0] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkLeanAmountApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinMaxAngle "Max Angle         " range:[0,180,75] offset:[160,-41] type:#integer fieldwidth:45 align:#left
	checkbutton chkMaxAngleApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	spinner spinLeanRand "Random Factor " range:[0,1,0] offset:[160,5] type:#float fieldwidth:45 align:#left
	checkbutton chkLeanRandApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty " " height:1
)

button btnSelectApplyLeanAll "+" width:18 height:18 align:#left offset:[254,-73]
button btnSelectApplyLeanNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyLeanInvert "/" width:18 height:18 align:#left offset:[290,-23]

label lblEmpty2 " " offset:[0,42] height:1

group "Bending" (

	checkbutton chkBendEnableApply offset:[0,1] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkBendEnable "Enabled" offset:[12,-17] align:#left checked:true

	spinner spinBendAmount "Amount              " range:[-1,1,0] offset:[0,5] type:#float fieldwidth:45 align:#left
	checkbutton chkBendAmountApply offset:[138,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	checkbutton chkBendStyleApply offset:[160,-41] width:10 height:10 align:#left highlightcolor:CheckedColor
	checkbox chkBendStyle "Use Leaning" offset:[172,-17] align:#left checked:true

	spinner spinBendRand "Random Factor " range:[0,1,0] offset:[160,5] type:#float fieldwidth:45 align:#left
	checkbutton chkBendRandApply offset:[296,-18] width:10 height:10 align:#left highlightcolor:CheckedColor

	label lblEmpty3 " " height:1
)

button btnSelectApplyBendAll "+" width:18 height:18 align:#left offset:[254,-73]
button btnSelectApplyBendNone "-" width:18 height:18 align:#left offset:[272,-23]
button btnSelectApplyBendInvert "/" width:18 height:18 align:#left offset:[290,-23]


on btnSelectApplyAll pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 5 ) do
			desc[j][6].state = on
)

on btnSelectApplyNone pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 5 ) do
			desc[j][6].state = off
)

on btnSelectApplyInvert pressed do (
	local j

	for j=1 to desc.count where
		( classof desc[j][6] == checkbuttoncontrol ) and
		( desc[j][4] == 5 ) do
			desc[j][6].state = not desc[j][6].state
)


on btnSelectApplyLeanAll pressed do (

	chkLeanEnableApply.state = on
	chkLeanAmountApply.state = on
	chkMaxAngleApply.state = on
	chkLeanRandApply.state = on
)


on btnSelectApplyLeanNone pressed do (

	chkLeanEnableApply.state = off
	chkLeanAmountApply.state = off
	chkMaxAngleApply.state = off
	chkLeanRandApply.state = off
)


on btnSelectApplyLeanInvert pressed do (

	chkLeanEnableApply.state = not chkLeanEnableApply.state
	chkLeanAmountApply.state = not chkLeanAmountApply.state
	chkMaxAngleApply.state = not chkMaxAngleApply.state
	chkLeanRandApply.state = not chkLeanRandApply.state
)


on btnSelectApplyBendAll pressed do (

	chkBendEnableApply.state = on
	chkBendAmountApply.state = on
	chkBendStyleApply.state = on
	chkBendRandApply.state = on
)


on btnSelectApplyBendNone pressed do (

	chkBendEnableApply.state = off
	chkBendAmountApply.state = off
	chkBendStyleApply.state = off
	chkBendRandApply.state = off
)


on btnSelectApplyBendInvert pressed do (

	chkBendEnableApply.state = not chkBendEnableApply.state
	chkBendAmountApply.state = not chkBendAmountApply.state
	chkBendStyleApply.state = not chkBendStyleApply.state
	chkBendRandApply.state = not chkBendRandApply.state
)

)


-------------------------- initialization -------------------------------

if ShagFur == undefined then (
	messagebox "Shag: Hair <-> MAXScript link not found, probably missing ShagBase.dlx"
) else (
	floatShagMgr = newRolloutFloater "Shag Manager" 355 600

	addrollout rolMgr floatShagMgr
	addrollout rolCutters floatShagMgr rolledUp:true
	addrollout rolParam floatShagMgr rolledUp:true
	addrollout rolShade floatShagMgr rolledUp:true
	addrollout rolDyn floatShagMgr rolledUp:true
	addrollout rolLean floatShagMgr rolledUp:true
)

------ initialize arrays

-- desc columns:
-- 1 - property name
-- 2 - section
-- 3 - key
-- 4 - rollout
-- 5 - control
-- 6 - active button
-- 7 - relative
-- 8 - is of time type
-- 9 - 1=fur, 2=hair, 3=both

	desc =
#(	-- property name						section			key						rollout		control						active button				relative  time  fur/hair
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#( #Use_Restricting_Cutters,               "Cutters",		"Use",						1, rolCutters.chkRestrCutsEnabled,	rolCutters.chkRestrCutsEnabledApply,0,	0,	3 ),
#( #Use_Modeling_Cutters,                  "Model Cutters",	"Use",						1, rolCutters.chkModCutsEnabled,	rolCutters.chkModCutsEnabledApply,	0,	0,	3 ),

#( #length,                                "Length",		"Maximum",					2, rolParam.spinLenMax,             rolParam.chkLenMaxApply,			1,	0,	3 ),
#( #Length_Random_Factor,                  "Length",		"Random",					2, rolParam.spinLenRand,			rolParam.chkLenRandApply,			1,	0,	3 ),
#( #Use_Length_SubMat,                     "Length",		"UseSubMat",				2, rolParam.chkLenFromMat,			rolParam.chkLenFromMatApply,		0,	0,	3 ),
#( #Length_SubMat_ID,                      "Length",		"SubMat",					2, rolParam.spinLenMat,				rolParam.chkLenMatApply,			0,	0,	3 ),

#( #Density_Type,                          "Density",		"Type",						2, rolParam.radioDensityType,		rolParam.chkDensityTypeApply,		0,	0,	3 ),
#( #Density_Absolute,                      "Density",		"Absolute",					2, rolParam.spinDensAbs,			rolParam.chkDensAbsApply,			1,	0,	3 ),
#( #Density_Per_Area,                      "Density",		"PerArea",					2, rolParam.spinDensPerArea,		rolParam.chkDensPerAreaApply,		1,	0,	3 ),
#( #Use_Density_SubMat,                    "Density",		"UseSubMat",				2, rolParam.chkDensFromMat,			rolParam.chkDensFromMatApply,		0,	0,	3 ),
#( #Density_SubMat_ID,                     "Density",		"SubMat",					2, rolParam.spinDensMat,			rolParam.chkDensMatApply,			0,	0,	3 ),

#( #thickness,                             "Thickness",		"Maximum",					2, rolParam.spinThickMax,			rolParam.chkThickMaxApply,			1,	0,	3 ),
#( #Thickness_Random_Factor,               "Thickness",		"Random",					2, rolParam.spinThickRand,			rolParam.chkThickRandApply,			1,	0,	3 ),
#( #Thickness_Link_Random_To_Length,       "Thickness",		"LinkThickRand2LengthRand",	2, rolParam.chkLinkToLenRand,		rolParam.chkLinkToLenRandApply,		0,	0,	3 ),
#( #Shadow_Thickness_Factor,               "Thickness",		"ShadowFactor",				2, rolParam.spinShadowFactor,		rolParam.chkShadowFactorApply,		1,	0,	3 ),
#( #Thickness_Use_SubMat,                  "Thickness",		"UseSubMat",				2, rolParam.chkThickFromMat,		rolParam.chkThickFromMatApply,		0,	0,	3 ),
#( #Thickness_SubMat_ID,                   "Thickness",		"SubMat",					2, rolParam.spinThickMat,			rolParam.chkThickMatApply,			0,	0,	3 ),

#( #Clumping_Amount,                       "Clumping",		"Amount",					2, rolParam.spinClumpAmount,		rolParam.chkClumpAmountApply,		1,	0,	3 ),
#( #Clumping_Amount_Use_SubMat,            "Clumping",		"UseAmountSubMat",			2, rolParam.chkClumpAmountFromMat,	rolParam.chkClumpAmountFromMatApply,0,	0,	3 ),
#( #Clumping_Amount_SubMat_ID,             "Clumping",		"AmountSubMat",				2, rolParam.spinClumpAmountMat,		rolParam.chkClumpAmountMatApply,	0,	0,	3 ),
#( #Clumping_Radius,                       "Clumping",		"Radius",					2, rolParam.spinClumpRadius,		rolParam.chkClumpRadiusApply,		1,	0,	3 ),
#( #Clumping_Radius_Use_SubMat,            "Clumping",		"UseRadiusSubMat",			2, rolParam.chkClumpRadiusFromMat,	rolParam.chkClumpRadiusFromMatApply,0,	0,	3 ),
#( #Clumping_Radius_SubMat_ID,             "Clumping",		"RadiusSubMat",				2, rolParam.spinClumpRadiusMat,		rolParam.chkClumpRadiusMatApply,	0,	0,	3 ),
#( #Clumping_Attractors,                   "Clumping",		"Attractors",				2, rolParam.radioAttractors,		rolParam.chkAttractorsApply,		0,	0,	3 ),
#( #Clumping_Attractors_Percent,           "Clumping",		"Percent",					2, rolParam.spinClumpPercent,		rolParam.chkClumpPercentApply,		1,	0,	3 ),
#( #Clumping_Attractors_Absolute,          "Clumping",		"Absolute",					2, rolParam.spinClumpPerFace,		rolParam.chkClumpPerFaceApply,		1,	0,	3 ),
#( #Clumping_Attractors_Per_Area,          "Clumping",		"PerArea",					2, rolParam.spinClumpPerArea,		rolParam.chkClumpPerAreaApply,		1,	0,	3 ),

#( #Use_Curliness,                         "Curliness",		"Use",						2, rolParam.chkCurlEnabled,			rolParam.chkCurlEnabledApply,		0,	0,	3 ),
#( #Curliness_Radius,                      "Curliness",		"Radius",					2, rolParam.spinCurlRadius,			rolParam.chkCurlRadiusApply,		1,	0,	3 ),
#( #Curliness_Radius_Use_SubMat,           "Curliness",		"UseRadiusSubMat",			2, rolParam.chkCurlRadiusFromMat,	rolParam.chkCurlRadiusFromMatApply,	0,	0,	3 ),
#( #Curliness_Radius_SubMat_ID,            "Curliness",		"RadiusSubMat",				2, rolParam.spinCurlRadiusMat,		rolParam.chkCurlRadiusMatApply,		0,	0,	3 ),
#( #Curliness_Radius_Use_Curve,            "Curliness",		"UseRadiusCurve",			0, undefined,						undefined,							0,	0,	3 ),
#( #Curliness_Turns,                       "Curliness",		"Turns",					2, rolParam.spinCurlTurns,			rolParam.chkCurlTurnsApply,			1,	0,	3 ),
#( #Curliness_Turns_Use_SubMat,            "Curliness",		"UseTurnsSubMat",			2, rolParam.chkCurlTurnsFromMat,	rolParam.chkCurlTurnsFromMatApply,	0,	0,	3 ),
#( #Curliness_Turns_SubMat_ID,             "Curliness",		"TurnsSubMat",				2, rolParam.spinCurlTurnsMat,		rolParam.chkCurlTurnsMatApply,		0,	0,	3 ),
#( #Curliness_Turns_Use_Curve,             "Curliness",		"UseTurnsCurve",			0, undefined,						undefined,							0,	0,	3 ),

#( #Random_Seed,                           "Direction Randoms","RandSeed",				2, rolParam.spinRandSeed,			rolParam.chkRandSeedApply,			0,	0,	3 ),
#( #Middle_Strands_Direction_Random_Factor,"Direction Randoms","MiddleStrands",			2, rolParam.spinMiddleRand,			rolParam.chkMiddleRandApply,		1,	0,	3 ),
#( #Vertex_Strands_Direction_Random_Factor,"Direction Randoms","VertexStrands",			2, rolParam.spinVertexRand,			rolParam.chkVertexRandApply,		1,	0,	3 ),

#( #Quality_Memory,                        "Picture Quality","Quality",					3, rolShade.spinQuality,			rolShade.chkQualityApply,			0,	0,	3 ),
#( #Bezier_Steps,                          "Picture Quality","BezierSegs",				3, rolShade.spinBezSteps,			rolShade.chkBezStepsApply,			0,	0,	3 ),
#( #Viewport_Density,                      "Picture Quality","ViewDensity",				3, rolShade.spinViewDens,			rolShade.chkViewDensApply,			0,	0,	3 ),
#( #Knots,                                 "Picture Quality","Knots",					3, rolShade.spinKnots,				rolShade.chkKnotsApply,				0,	0,	3 ),

#( #Fix_Shape,                             "Shape",			"FixShape",				2, rolParam.chkFixShapeToEmitter,	rolParam.chkFixShapeToEmitterApply,	0,	0,	3 ),
#( #Fixing_Reference_Frame,                "Shape",			"ReferenceFrame",			2, rolParam.spinRefFrame,			rolParam.chkRefFrameApply,			0,	1,	3 ),
#( #Cross_Section,                         "Shape",			"CrossSection",				2, rolParam.radioSection,			rolParam.chkSectionApply,			0,	0,	3 ),
#( #Orientation_Axis,                      "Shape",			"Axis",						2, rolParam.radioOrientAxis,		rolParam.chkOrientAxisApply,		0,	0,	3 ),
#( #Orientation_Angle,                     "Shape",			"Angle",					2, rolParam.spinOrientAngle,		rolParam.chkOrientAngleApply,		1,	0,	3 ),
#( #Orientation_Random_Factor,             "Shape",			"Random",					2, rolParam.spinOrientRand,			rolParam.chkOrientRandApply,		1,	0,	3 ),
#( #Use_Variable_Thickness,                "Shape",			"UseVariableThickness",		0, undefined,						undefined,							0,	0,	3 ),

#( #Use_Geometry,                          "Geometry",		"Use",						3, rolShade.chkGeomEnable,			rolShade.chkGeomEnableApply,		0,	0,	3 ),
#( #Geometry_Smooth,                       "Geometry",		"Smooth",					3, rolShade.chkGeomSmooth,			rolShade.chkGeomSmoothApply,		0,	0,	3 ),
#( #Geometry_Sides,                        "Geometry",		"Sides",					3, rolShade.spinGeomSides,			rolShade.chkGeomSidesApply,			0,	0,	3 ),
#( #Geometry_Min_Thickness,                "Geometry",		"MinThickness",				3, rolShade.spinGeomMinThick,		rolShade.chkGeomMinThickApply,		1,	0,	3 ),
#( #Geometry_Thin_Handling,                "Geometry",		"GeomThinHandling",			3, rolShade.radioGeomThin,			rolShade.chkGeomThinApply,			0,	0,	3 ),

#( #Base_Color,                            "Shading",		"BaseColor",				3, rolShade.colBase,				rolShade.chkColBaseApply,			0,	0,	3 ),
#( #Tip_Color,                             "Shading",		"TipColor",					3, rolShade.colTip,					rolShade.chkColTipApply,			0,	0,	3 ),
#( #Base_Color_Type,                       "Shading",		"BaseColorUseSubMat",		3, rolShade.chkBaseFromMat,			rolShade.chkBaseFromMatApply,		0,	0,	3 ),
#( #Tip_Color_Type,                        "Shading",		"TipColorUseSubMat",		3, rolShade.chkTipFromMat,			rolShade.chkTipFromMatApply,		0,	0,	3 ),
#( #Base_Color_SubMat_ID,                  "Shading",		"BaseColorSubMat",			3, rolShade.spinBaseMat,			rolShade.chkBaseMatApply,			0,	0,	3 ),
#( #Tip_Color_SubMat_ID,                   "Shading",		"TipColorSubMat",			3, rolShade.spinTipMat,				rolShade.chkTipMatApply,			0,	0,	3 ),
#( #Color_Bias,                            "Shading",		"ColBias",					3, rolShade.spinColBias,			rolShade.chkColBiasApply,			1,	0,	3 ),
#( #Color_Random_Factor,                   "Shading",		"Random",					3, rolShade.spinColRand,			rolShade.chkColRandApply,			1,	0,	3 ),
#( #Shade_Bezier_Segments,                 "Shading",		"ShadeBezSegs",				3, rolShade.chkShadeBezSegs,		rolShade.chkShadeBezSegsApply,		0,	0,	3 ),
#( #Shade_Middle_Strands,                  "Shading",		"ShadeMidStrands",			3, rolShade.chkShadeMid,			rolShade.chkShadeMidApply,			0,	0,	3 ),
#( #Max_Width_Samples,                     "Shading",		"WidthSamples",				3, rolShade.spinWidthSamples,		rolShade.chkWidthSamplesApply,		0,	0,	3 ),
#( #Width_Filtering,                       "Shading",		"WidthFiltering",			3, rolShade.chkFiltering,			rolShade.chkFilteringApply,			0,	0,	3 ),
#( #Material_ID_Usage,                     "Shading",		"MaterialIDUsage",			3, rolShade.radioMatIDType,			rolShade.chkMatIDTypeApply,			0,	0,	3 ),
#( #Material_ID,                           "Shading",		"MaterialID",				3, rolShade.spinMatID,				rolShade.chkMatIDApply,				0,	0,	3 ),

#( #Use_Dynamics,                          "Dynamics",		"Use",						4, rolDyn.radioDynUsage,			rolDyn.chkDynUsageApply,			0,	0,	3 ),
#( #Dyn_Method,                            "Dynamics",		"Method",					4, rolDyn.radioDynMethod,			rolDyn.chkDynMethodApply,			0,	0,	3 ),
#( #Dyn_Multithreading,                    "Dynamics",		"UseMultithreading",		0, undefined,						undefined,							0,	0,	3 ),
#( #Dyn_Start_Frame,                       "Dynamics",		"Start",					4, rolDyn.spinStartFrame,			rolDyn.chkStartFrameApply,			0,	1,	3 ),
#( #Dyn_End_Frame,                         "Dynamics",		"End",						4, rolDyn.spinEndFrame,				rolDyn.chkEndFrameApply,			0,	1,	3 ),
#( #Dyn_Key_Type,                          "Dynamics",		"KeyType",                  4, rolDyn.radioKeyType,				rolDyn.chkKeyTypeApply,				0,	0,	3 ),
#( #Dyn_Key_Threshold,                     "Dynamics",		"KeyThreshold",				4, rolDyn.spinKeyThresh,			rolDyn.chkKeyThreshApply,			0,	0,	3 ),
#( #Dyn_Simple_Elasticity,                 "Dynamics",		"SimpleElasticity",			4, rolDyn.spinSimpElast,			rolDyn.chkSimpElast,				1,	0,	3 ),
#( #Dyn_Collision_Friction,                "Dynamics",		"CollisionFriction",		4, rolDyn.spinColFrict,				rolDyn.chkColFrictApply,			1,	0,	3 ),
#( #Dyn_Collision_Static_Friction,         "Dynamics",		"CollisionStaticFriction",  4, rolDyn.spinColStaticFrict,       rolDyn.chkColStaticFrictApply,		1,	0,	3 ),
#( #Dyn_Collision_Distance,                "Dynamics",		"CollisionDistance",		4, rolDyn.spinColDist,				rolDyn.chkColDistApply,				1,	0,	3 ),
#( #Dyn_Collision_Distance_Random_Factor,  "Dynamics",		"CollisionDistanceRandomFactor",4, rolDyn.spinColDistRand,		rolDyn.chkColDistRandApply,			1,	0,	3 ),
#( #Dyn_Mass,                              "Dynamics",		"Mass",						4, rolDyn.spinMass,					rolDyn.chkMassApply,				1,	0,	3 ),
#( #Dyn_Use_Winds,                         "Dynamics",		"UseWinds",					4, rolDyn.chkWindsEnable,			rolDyn.chkWindsEnableApply,			0,	0,	3 ),
#( #Dyn_Use_Gravitys,                      "Dynamics",		"UseGravs",					4, rolDyn.chkGravsEnable,			rolDyn.chkGravsEnableApply,			0,	0,	3 ),
#( #Dyn_Use_Collisions,                    "Dynamics",		"UseCollisions",			4, rolDyn.chkColsEnable,			rolDyn.chkColsEnableApply,			0,	0,	3 ),
#( #Dyn_Get_Out_Of_Colliders_At_Start,     "Dynamics",      "GetOutOfCollidersAtStart",	4, rolDyn.chkGetOut,				rolDyn.chkGetOutApply,				0,	0,	3 ),
#( #Dyn_Max_Velocity,                      "Dynamics",		"MaxVelocity",				4, rolDyn.spinMaxVelo,				rolDyn.chkMaxVeloApply,				1,	0,	3 ),
#( #Dyn_Edge_Collisions,                   "Dynamics",      "EdgeCollisions",			4, rolDyn.chkSegColls,				rolDyn.chkSegCollsApply,			0,	0,	3 ),

#( #Use_Leaning,                           "Leaning",		"Use",						5, rolLean.chkLeanEnable,			rolLean.chkLeanEnableApply,			0,	0,	1 ),
#( #Lean_Amount,                           "Leaning",		"Amount",					5, rolLean.spinLeanAmount,			rolLean.chkLeanAmountApply,			1,	0,	1 ),
#( #Lean_Amount_Random_Factor,             "Leaning",		"Random",					5, rolLean.spinLeanRand,			rolLean.chkLeanRandApply,			1,	0,	1 ),
#( #Lean_Max_Angle,                        "Leaning",		"MaxAngle",					5, rolLean.spinMaxAngle,			rolLean.chkMaxAngleApply,			0,	0,	1 ),

#( #Use_Bending,                           "Bending",		"Use",						5, rolLean.chkBendEnable,			rolLean.chkBendEnableApply,			0,	0,	1 ),
#( #Bend_Amount,                           "Bending",		"Amount",					5, rolLean.spinBendAmount,			rolLean.chkBendAmountApply,			1,	0,	1 ),
#( #Bend_Amount_Random_Factor,             "Bending",		"Random",					5, rolLean.spinBendRand,			rolLean.chkBendRandApply,			1,	0,	1 ),
#( #Bend_Style,                            "Bending",		"Type",						5, rolLean.chkBendStyle,			rolLean.chkBendStyleApply,			0,	0,	1 ),

#( #Auto_Influence_Mixing,				   "Model Hairs",	"AutoInfMixing",			1, rolCutters.spinAutoInfMix,		rolCutters.chkAutoInfMixApply,		0,	0,	2 ) )

pRelValues = #()
for i=1 to desc.count where desc[i][4] > 0 and desc[i][7]==1 do
	append pRelValues 1

